// automatically generated by the FlatBuffers compiler, do not modify


#ifndef FLATBUFFERS_GENERATED_ENUMS_H_
#define FLATBUFFERS_GENERATED_ENUMS_H_

#include "flatbuffers/flatbuffers.h"

// Ensure the included flatbuffers.h is the same version as when this file was
// generated, otherwise it may not be compatible.
static_assert(FLATBUFFERS_VERSION_MAJOR == 24 &&
              FLATBUFFERS_VERSION_MINOR == 12 &&
              FLATBUFFERS_VERSION_REVISION == 23,
             "Non-compatible flatbuffers version included");

enum Color : int32_t {
  Color_Red = 1,
  Color_Blue = 2,
  Color_Orange = 3
};
enum Size : int32_t {
  Size_Small = 10,
  Size_Large = 100,
  Size_Medium = 1000
};
#endif  // FLATBUFFERS_GENERATED_ENUMS_H_
