// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.OtherNameSpace;

import java.nio.*;
import java.lang.*;
import java.util.*;
import com.google.flatbuffers.*;

public class TableBT {
  private TableAT a;

  public TableAT getA() { return a; }

  public void setA(TableAT a) { this.a = a; }


  public TableBT() {
    this.a = null;
  }
}

