// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

export enum LongEnum {
  LongOne = '2',
  LongTwo = '4',
  LongBig = '1099511627776'
}
