// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */
export var Abc;
(function (Abc) {
    Abc[Abc["a"] = 0] = "a";
})(Abc || (Abc = {}));
