// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */
export var class_;
(function (class_) {
    class_[class_["arguments_"] = 0] = "arguments_";
})(class_ || (class_ = {}));
