// automatically generated by the FlatBuffers compiler, do not modify

package MyGame.OtherNameSpace;

import java.nio.*;
import java.lang.*;
import java.util.*;
import com.google.flatbuffers.*;

public class UnusedT {
  private int a;

  public int getA() { return a; }

  public void setA(int a) { this.a = a; }


  public UnusedT() {
    this.a = 0;
  }
}

