// automatically generated by the FlatBuffers compiler, do not modify

package NamespaceA.NamespaceB;

import java.nio.*;
import java.lang.*;
import java.util.*;
import com.google.flatbuffers.*;

public class TableInNestedNST {
  private int foo;

  public int getFoo() { return foo; }

  public void setFoo(int foo) { this.foo = foo; }


  public TableInNestedNST() {
    this.foo = 0;
  }
}

