/**
 * Composite components of Monster color.
 */
export declare enum Color {
    Red = 1,
    /**
     * \brief color Green
     * Green is bit_flag with value (1u << 1)
     */
    Green = 2,
    /**
     * \brief color Blue (1u << 3)
     */
    Blue = 8
}
