// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */
export var OptionalByte;
(function (OptionalByte) {
    OptionalByte[OptionalByte["None"] = 0] = "None";
    OptionalByte[OptionalByte["One"] = 1] = "One";
    OptionalByte[OptionalByte["Two"] = 2] = "Two";
})(OptionalByte || (OptionalByte = {}));
