/**
 * New schema language features that are not supported by old code generators.
 */
export declare enum AdvancedFeatures {
    AdvancedArrayFeatures = "1",
    AdvancedUnionFeatures = "2",
    OptionalScalars = "4",
    DefaultVectorsAndStrings = "8"
}
