// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */
export var TestEnum;
(function (TestEnum) {
    TestEnum[TestEnum["A"] = 0] = "A";
    TestEnum[TestEnum["B"] = 1] = "B";
    TestEnum[TestEnum["C"] = 2] = "C";
})(TestEnum || (TestEnum = {}));
