// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */
export var BaseType;
(function (BaseType) {
    BaseType[BaseType["None"] = 0] = "None";
    BaseType[BaseType["UType"] = 1] = "UType";
    BaseType[BaseType["Bool"] = 2] = "Bool";
    BaseType[BaseType["Byte"] = 3] = "Byte";
    BaseType[BaseType["UByte"] = 4] = "UByte";
    BaseType[BaseType["Short"] = 5] = "Short";
    BaseType[BaseType["UShort"] = 6] = "UShort";
    BaseType[BaseType["Int"] = 7] = "Int";
    BaseType[BaseType["UInt"] = 8] = "UInt";
    BaseType[BaseType["Long"] = 9] = "Long";
    BaseType[BaseType["ULong"] = 10] = "ULong";
    BaseType[BaseType["Float"] = 11] = "Float";
    BaseType[BaseType["Double"] = 12] = "Double";
    BaseType[BaseType["String"] = 13] = "String";
    BaseType[BaseType["Vector"] = 14] = "Vector";
    BaseType[BaseType["Obj"] = 15] = "Obj";
    BaseType[BaseType["Union"] = 16] = "Union";
    BaseType[BaseType["Array"] = 17] = "Array";
    BaseType[BaseType["Vector64"] = 18] = "Vector64";
    BaseType[BaseType["MaxBaseType"] = 19] = "MaxBaseType";
})(BaseType || (BaseType = {}));
