// automatically generated by the FlatBuffers compiler, do not modify
export class FallingTub {
    constructor() {
        this.bb = null;
        this.bb_pos = 0;
    }
    __init(i, bb) {
        this.bb_pos = i;
        this.bb = bb;
        return this;
    }
    weight() {
        return this.bb.readInt32(this.bb_pos);
    }
    mutate_weight(value) {
        this.bb.writeInt32(this.bb_pos + 0, value);
        return true;
    }
    static getFullyQualifiedName() {
        return 'FallingTub';
    }
    static sizeOf() {
        return 4;
    }
    static createFallingTub(builder, weight) {
        builder.prep(4, 4);
        builder.writeInt32(weight);
        return builder.offset();
    }
    unpack() {
        return new FallingTubT(this.weight());
    }
    unpackTo(_o) {
        _o.weight = this.weight();
    }
}
export class FallingTubT {
    constructor(weight = 0) {
        this.weight = weight;
    }
    pack(builder) {
        return FallingTub.createFallingTub(builder, this.weight);
    }
}
