// automatically generated by the FlatBuffers compiler, do not modify
export class BookReader {
    constructor() {
        this.bb = null;
        this.bb_pos = 0;
    }
    __init(i, bb) {
        this.bb_pos = i;
        this.bb = bb;
        return this;
    }
    booksRead() {
        return this.bb.readInt32(this.bb_pos);
    }
    mutate_books_read(value) {
        this.bb.writeInt32(this.bb_pos + 0, value);
        return true;
    }
    static getFullyQualifiedName() {
        return 'BookReader';
    }
    static sizeOf() {
        return 4;
    }
    static createBookReader(builder, books_read) {
        builder.prep(4, 4);
        builder.writeInt32(books_read);
        return builder.offset();
    }
    unpack() {
        return new BookReaderT(this.booksRead());
    }
    unpackTo(_o) {
        _o.booksRead = this.booksRead();
    }
}
export class BookReaderT {
    constructor(booksRead = 0) {
        this.booksRead = booksRead;
    }
    pack(builder) {
        return BookReader.createBookReader(builder, this.booksRead);
    }
}
