--[[ MyGame.Example.AnyAmbiguousAliases

  Automatically generated by the FlatBuffers compiler, do not modify.
  Or modify. I'm a message, not a cop.

  flatc version: 24.12.23

  Declared by  : //monster_test.fbs
  Rooting type : MyGame.Example.Monster (//monster_test.fbs)

--]]

local AnyAmbiguousAliases = {
  NONE = 0,
  M1 = 1,
  M2 = 2,
  M3 = 3,
}

return AnyAmbiguousAliases