// automatically generated by the FlatBuffers compiler, do not modify

import java.nio.*;
import java.lang.*;
import java.util.*;
import com.google.flatbuffers.*;

public class TableAT {
  private MyGame.OtherNameSpace.TableBT b;

  public MyGame.OtherNameSpace.TableBT getB() { return b; }

  public void setB(MyGame.OtherNameSpace.TableBT b) { this.b = b; }


  public TableAT() {
    this.b = null;
  }
}

