// automatically generated by the FlatBuffers compiler, do not modify

/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */

export enum Race {
  None = -1,
  Human = 0,
  Dwarf = 1,
  Elf = 2
}
