// automatically generated by the FlatBuffers compiler, do not modify
/* eslint-disable @typescript-eslint/no-unused-vars, @typescript-eslint/no-explicit-any, @typescript-eslint/no-non-null-assertion */
/**
 * Composite components of Monster color.
 */
export var Color;
(function (Color) {
    Color[Color["Red"] = 1] = "Red";
    /**
     * \brief color Green
     * Green is bit_flag with value (1u << 1)
     */
    Color[Color["Green"] = 2] = "Green";
    /**
     * \brief color Blue (1u << 3)
     */
    Color[Color["Blue"] = 8] = "Blue";
})(Color || (Color = {}));
