// automatically generated by the FlatBuffers compiler, do not modify
export class Unused {
    constructor() {
        this.bb = null;
        this.bb_pos = 0;
    }
    __init(i, bb) {
        this.bb_pos = i;
        this.bb = bb;
        return this;
    }
    a() {
        return this.bb.readInt32(this.bb_pos);
    }
    mutate_a(value) {
        this.bb.writeInt32(this.bb_pos + 0, value);
        return true;
    }
    static getFullyQualifiedName() {
        return 'MyGame.OtherNameSpace.Unused';
    }
    static sizeOf() {
        return 4;
    }
    static createUnused(builder, a) {
        builder.prep(4, 4);
        builder.writeInt32(a);
        return builder.offset();
    }
    unpack() {
        return new UnusedT(this.a());
    }
    unpackTo(_o) {
        _o.a = this.a();
    }
}
export class UnusedT {
    constructor(a = 0) {
        this.a = a;
    }
    pack(builder) {
        return Unused.createUnused(builder, this.a);
    }
}
