// automatically generated by the FlatBuffers compiler, do not modify

package NamespaceA.NamespaceB;

import java.nio.*;
import java.lang.*;
import java.util.*;
import com.google.flatbuffers.*;

public class StructInNestedNST {
  private int a;
  private int b;

  public int getA() { return a; }

  public void setA(int a) { this.a = a; }

  public int getB() { return b; }

  public void setB(int b) { this.b = b; }


  public StructInNestedNST() {
    this.a = 0;
    this.b = 0;
  }
}

